__all__ = ['LEAGUE_LISTS']

LEAGUE_LISTS = [
    {'caption': 'European Championships France ', 'league': 'EC'},
    {'caption': 'Premier League', 'league': 'PL'},
    {'caption': 'Championship', 'league': 'ELC'},
    {'caption': 'League One', 'league': 'EL1'},
    {'caption': 'FA-Cup', 'league': 'FAC'},
    {'caption': '1. Bundesliga', 'league': 'BL1'},
    {'caption': '2. Bundesliga', 'league': 'BL2'},
    {'caption': 'DFB-Pokal', 'league': 'DFB'},
    {'caption': 'Eredivisie', 'league': 'DED'},
    {'caption': 'Ligue 1', 'league': 'FL1'},
    {'caption': 'Ligue 2', 'league': 'FL2'},
    {'caption': 'Primera Division', 'league': 'PD'},
    {'caption': 'Liga Adelante', 'league': 'SD'},
    {'caption': 'Serie A', 'league': 'SA'},
    {'caption': 'Primeira Liga', 'league': 'PPL'},
    {'caption': 'Champions League', 'league': 'CL'},
    {'caption': 'Serie B', 'league': 'SB'},
    {'caption': 'English National League', 'league': 'ENL'},
    {'caption': 'League Two', 'league': 'EL2'}
]
