from .leagueids import LEAGUE_IDS
from .leaguelists import LEAGUE_LISTS
from .leaguenames import LEAGUE_NAMES
from .leagueproperties import LEAGUE_PROPERTIES
from .teamnames import TEAM_NAMES
from .config import config
from .urls import urls
